package senyuan.tiandao.flutter_wilddog

import android.app.Activity
import android.util.Log
import com.wilddog.client.*
import com.wilddog.wilddogcore.WilddogApp
import com.wilddog.wilddogcore.WilddogOptions
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar


class FlutterWilddogPlugin : MethodCallHandler {
    constructor(activity: Activity, channel: MethodChannel) {
        this.activity = activity
        this.channel = channel
    }

    private var channel: MethodChannel
    private var activity: Activity
    private lateinit var wilddog: SyncReference

    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "flutter_wilddog")
            channel.setMethodCallHandler(FlutterWilddogPlugin(registrar.activity(),channel))
        }
    }


    override fun onMethodCall(call: MethodCall, result: Result) {
        Log.e("method", call.method.toString())
        if (call.arguments != null) Log.e("arguments", call.arguments.toString())

        when (call.method) {


            "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")


            "init" -> {
                // 初始化

                val options = WilddogOptions.Builder().setSyncUrl("https://${call.argument<String>("app_id")}.wilddogio.com").build()

                try {
                    WilddogApp.initializeApp(activity.applicationContext, options)
                    var sync = WilddogSync.getInstance()
                    sync.setPersistenceEnabled(true)
                } catch (e: Exception) {
                }
                var sync = WilddogSync.getInstance()
                wilddog = sync.reference

                result.success(null)
            }


            "setValue" -> {
                wilddog.child(call.argument("path")).setValue(call.argument("data")) { error, ref ->
                    if (error != null) {
                        Log.d("error", error!!.toString())
                        result.error(error.details, error.message, error)
                    } else {
                        Log.d("success", "setValue success")
                        result.success("")
                    }
                }
            }

            "removeValue" -> {
                wilddog.child(call.argument("path")).removeValue { syncError, syncReference ->
                    if (syncError != null) {
                        Log.d("error", syncError!!.toString())
                        result.error(syncError.details, syncError.message, syncError)
                    } else {
                        Log.d("success", "removeValue success")
                        result.success("setValue success")
                    }

                }
            }

            "addValueEventListener" -> {
                // dataSnapshot 里面的数据会一直和云端保持同步
                wilddog.child(call.argument("path")).addValueEventListener(object : ValueEventListener {
                    override fun onDataChange(dataSnapshot: DataSnapshot) {
                        if (dataSnapshot.exists()) {
                            Log.d("onDataChange", dataSnapshot.toString())

                            val data = mapOf(dataSnapshot.key to dataSnapshot.value)

                            channel.invokeMethod("onValueEvent", data)
                        }
                    }

                    override fun onCancelled(syncError: SyncError?) {
                        if (syncError != null) {
                            Log.d("onCancelled", syncError.toString())
                        }
                    }
                })
                result.success(null)

            }
            "addListenerForSingleValueEvent" -> {
                // 如果你只想监听一次，那么你可以使用addListenerForSingleValueEvent()
                wilddog.child(call.argument("path")).addListenerForSingleValueEvent(object : ValueEventListener {
                    override fun onDataChange(dataSnapshot: DataSnapshot) {
                        if (dataSnapshot.value != null) {
                            Log.d("onDataChange", dataSnapshot.toString())

                            val data = mapOf(dataSnapshot.key to dataSnapshot.value)

                            channel.invokeMethod("onSingleValueEvent", data)
                        }
                    }

                    override fun onCancelled(syncError: SyncError?) {
                        if (syncError != null) {
                            Log.d("onCancelled", syncError.toString())
                        }
                    }
                })
                result.success(null)
            }

            else -> result.notImplemented()
        }
    }
}
