import 'dart:async';

import 'package:flutter/services.dart';

class FlutterWilddog {
  static const MethodChannel _channel = const MethodChannel('flutter_wilddog');

  //android 端 根结点删除时，无法接受到 数据事件
  static StreamController<Map> _valueEventStreamController = StreamController.broadcast();
  static Stream<Map> get valueEventStream  => _valueEventStreamController.stream;

  static StreamController<Map> _singleValueEventStreamController = StreamController.broadcast();
  static Stream<Map> get singleValueEventStream  => _singleValueEventStreamController.stream;


  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static Future<void> init(String app_id) async {
    _channel.setMethodCallHandler(handle);
     await _channel.invokeMethod('init', {'app_id':app_id});
  }

  static Future<String> setValue(String path, Map data) async {
    return await _channel.invokeMethod<String>('setValue', {'path':path, 'data':data});
  }

  static Future removeValue(String path) async {
    return await _channel.invokeMapMethod('removeValue', {'path':path});
  }

  static Future<String> addValueEventListener(String path) async {
    final String version =
        await _channel.invokeMethod('addValueEventListener', {'path':path});
    return version;
  }

  static Future<String> addListenerForSingleValueEvent(String path) async {
    final String version =
        await _channel.invokeMethod('addListenerForSingleValueEvent', {'path':path});
    return version;
  }



  static Future handle(MethodCall call) {
    switch(call.method){
      case  "onValueEvent":{
        _valueEventStreamController.add(call.arguments);
        break;
      }
      case  "onSingleValueEvent":{
        _singleValueEventStreamController.add(call.arguments);
        break;
      }

    }

    return Future.value();
  }
}
